package java.time;

import java.io.DataInput;
import java.io.IOException;
import java.io.DataOutput;
import java.io.InvalidObjectException;
import java.io.ObjectInputStream;
import java.time.zone.ZoneRulesException;
import java.time.zone.ZoneRulesProvider;
import java.util.Objects;
import java.time.zone.ZoneRules;
import java.io.Serializable;

final class ZoneRegion extends ZoneId implements Serializable
{
    private static final long serialVersionUID = 8386373296231747096L;
    private final String id;
    private final transient ZoneRules rules;
    
    static ZoneRegion ofId(String s, boolean b) {
        Objects.requireNonNull(s, "zoneId");
        checkName(s);
        ZoneRules rules = null;
        try {
            rules = ZoneRulesProvider.getRules(s, true);
        }
        catch (ZoneRulesException ex) {
            if (b) {
                throw ex;
            }
        }
        return new ZoneRegion(s, rules);
    }
    
    private static void checkName(String s) {
        int length = s.length();
        if (length < 2) {
            throw new DateTimeException("Invalid ID for region-based ZoneId, invalid format: " + s);
        }
        for (int i = 0; i < length; ++i) {
            int char1 = s.charAt(i);
            if ((char1 < 'a' || char1 > 'z') && (char1 < 'A' || char1 > 'Z') && (char1 != '/' || i == 0) && (char1 < '0' || char1 > '9' || i == 0) && (char1 != '~' || i == 0) && (char1 != '.' || i == 0) && (char1 != '_' || i == 0) && (char1 != '+' || i == 0) && (char1 != '-' || i == 0)) {
                throw new DateTimeException("Invalid ID for region-based ZoneId, invalid format: " + s);
            }
        }
        return;
    }
    
    ZoneRegion(String id, ZoneRules rules) {
        this.id = id;
        this.rules = rules;
    }
    
    @Override
    public String getId() {
        return this.id;
    }
    
    @Override
    public ZoneRules getRules() {
        return (this.rules != null) ? this.rules : ZoneRulesProvider.getRules(this.id, false);
    }
    
    private Object writeReplace() {
        return new Ser((byte)7, this);
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws InvalidObjectException {
        throw new InvalidObjectException("Deserialization via serialization delegate");
    }
    
    @Override
    void write(DataOutput dataOutput) throws IOException {
        dataOutput.writeByte(7);
        this.writeExternal(dataOutput);
    }
    
    void writeExternal(DataOutput dataOutput) throws IOException {
        dataOutput.writeUTF(this.id);
    }
    
    static ZoneId readExternal(DataInput dataInput) throws IOException {
        return ZoneId.of(dataInput.readUTF(), false);
    }
}
